@@ -53,6 +53,7 @@ import Control.Monad.Instances
 #endif
 import Control.Monad.Trans.Identity
 import Data.Functor.Identity
+import qualified Data.Functor.Sum as FSum
 import Data.List.NonEmpty hiding (map)
 import Data.Semigroup hiding (Product)
 import Data.Tagged
@@ -200,6 +201,19 @@ instance Comonad NonEmpty where
   extract ~(a :| _) = a
   {-# INLINE extract #-}
 
+coproduct :: (f a -> b) -> (g a -> b) -> FSum.Sum f g a -> b
+coproduct f _ (FSum.InL x) = f x
+coproduct _ g (FSum.InR y) = g y
+{-# INLINE coproduct #-}
+
+instance (Comonad f, Comonad g) => Comonad (FSum.Sum f g) where
+  extend f = coproduct
+               (FSum.InL . extend (f . FSum.InL))
+               (FSum.InR . extend (f . FSum.InR))
+  extract = coproduct extract extract
+  {-# INLINE extract #-}
+
+
 -- | @ComonadApply@ is to @Comonad@ like @Applicative@ is to @Monad@.
 --
 -- Mathematically, it is a strong lax symmetric semi-monoidal comonad on the
@@ -385,7 +399,7 @@ instance Applicative (Cokleisli w a) where
   Cokleisli f <*> Cokleisli a = Cokleisli (\w -> f w (a w))
 
 instance Monad (Cokleisli w a) where
-  return = Cokleisli . const
+  return = pure
   Cokleisli k >>= f = Cokleisli $ \w -> runCokleisli (f (k w)) w
 
 #if !(MIN_VERSION_base(4,7,0))
