@@ -5,6 +5,9 @@
 #elif __GLASGOW_HASKELL__ >= 702
 {-# LANGUAGE Trustworthy #-}
 #endif
+#ifndef MIN_VERSION_base
+#define MIN_VERSION_base(x,y,z) 1
+#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Comonad.Trans.Env
@@ -59,6 +62,9 @@ module Control.Comonad.Trans.Env
   , local
   ) where
 
+#if !(MIN_VERSION_base(4,8,0))
+import Control.Applicative
+#endif
 import Control.Comonad
 import Control.Comonad.Hoist.Class
 import Control.Comonad.Trans.Class
@@ -147,6 +153,10 @@ instance Comonad w => Comonad (EnvT e w) where
 instance ComonadTrans (EnvT e) where
   lower (EnvT _ wa) = wa
 
+instance (Monoid e, Applicative m) => Applicative (EnvT e m) where
+  pure = EnvT mempty . pure
+  EnvT ef wf <*> EnvT ea wa = EnvT (ef `mappend` ea) (wf <*> wa)
+
 -- | Gets rid of the environment. This differs from 'extract' in that it will
 --   not continue extracting the value from the contained comonad.
 lowerEnvT :: EnvT e w a -> w a
